// fft.h: interface for the Cxfft class.
//
//  This is a slightly modified version of Takuya OOURA's
//     original radix 4 FFT package.
//Copyright(C) 1996-1998 Takuya OOURA
//    (email: ooura@mmm.t.u-tokyo.ac.jp).
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_XFFT_H__C6E9A1D3_2D5F_11D3_A208_00A0C996E7F5__INCLUDED_)
#define AFX_XFFT_H__C6E9A1D3_2D5F_11D3_A208_00A0C996E7F5__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#define MAX_FFT_SIZE 0x80000
#define MIN_FFT_SIZE 0x400

#define WINDOW_RECTANGLE 0
#define WINDOW_HAMMING 1
#define WINDOW_HANNING 2

class Cxfft  
{
public:
	double GetSingleBin(INT type, INT bin);
	void SaveReference();
	void SetFFTParams( BOOL Complex, INT size,double dBmin, double dBmax, INT ave,
						INT postave,INT Window, BOOL invert, BOOL NCOnull,BOOL calcpeak,double maxdB);
	void ResetPeaks();
	void ResetFFT();
	void CalcFFT();
	BOOL GetDoubleFFTData( double*);
	BOOL GetFFTData( INT gain, INT start, INT stop, INT* OutBuf);
	BOOL GetFFTDeltaData( INT gain, INT start, INT stop, INT* OutBuf);
	BOOL GetFFTPeakData( INT gain, INT start, INT stop, INT* OutBuf);
	INT SendFFT(_int32* InBuf, INT OVLimit);
	INT SendFFT(SHORT* InBuf, INT OVLimit);
	double GetFFTave(int src,int start, int stop, double* peak);
	Cxfft();
	virtual ~Cxfft();
private:
	void FreeMemory();
	void makewt(int nw, int *ip, double *w);
	void makect(int nc, int *ip, double *c);
	void bitrv2(int n, int *ip, double *a);
	void cftfsub(int n, double *a, double *w);
	void rftfsub(int n, double *a, int nc, double *c);
	void CpxFFT(int n, double *a, double *w);
	void cft1st(int n, double *a, double *w);
	void cftmdl(int n, int l, double *a, double *w);

	BOOL m_SaveReference;
	BOOL m_NeedToSaveReference;
	BOOL m_NCOnull;
	BOOL m_Complex;
	BOOL m_Overload;
	BOOL m_Invert;
	INT m_MaxDelta;
	INT m_PostAverage;
	INT m_AveCount;
	INT m_TotalCount;
	INT m_LastAve;
	INT m_LastWindow;
	INT m_FFTSize;
	INT m_LastFFTSize;
	INT m_AveSize;
	double m_dBmaxOffset;
	double m_dBGainFactor;
	double m_dBDeltaOffset;
	double m_K_C;
	double m_K_B;
	double m_Gain;
	double m_MaxdB;
	INT* pWorkArea;
	double* pSinCosTbl;
	double* pWindowTbl;
	double* pFFTAveBuf;
	double* pFFTSumBuf;
	double* pFFTInBuf;
	double* pFFTSecBuf;
	CRITICAL_SECTION m_CriticalSection;	//for keeping threads from stomping on each other

};

#endif // !defined(AFX_XFFT_H__C6E9A1D3_2D5F_11D3_A208_00A0C996E7F5__INCLUDED_)
