// PlotData.cpp: implementation of the CPlotData class.
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@mindspring.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////

#include "stdafx.h"
#include "WaveVue.h"
#include "PlotData.h"
#include "WaveVueView.h"
#include "math.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CPlotData::CPlotData(CWaveVueDoc* pDoc)
{
	m_pDoc = pDoc;
//
	m_rLastPlot.bottom = 0;
	m_rLastPlot.top = 0;
	m_rLastPlot.left = 0;
	m_rLastPlot.right = 0;
}

CPlotData::~CPlotData()
{
}



//////////////////////////////////////////////////////////////////////
//  Called from the Windows view class when the window has changed
//      and needs to update the screen coordinates or "just because".
// if update is true, it also re plots the last data screen.
//////////////////////////////////////////////////////////////////////
void CPlotData::OnDrawPlot( CFormView* pView, CDC* pDC)
{
			//calculate new plot area coordinates
// Get pixel size of plot area bitmap
	ASSERT_VALID( pView );
	CWnd* pWnd = pView->GetDlgItem( IDC_PLOTAREA );
	ASSERT_VALID( pWnd);
	pWnd->GetWindowRect( m_rPlot );
	pView->ScreenToClient( &m_rPlot );
	m_rPlotsize.left = 0;
	m_rPlotsize.top = 0;
	m_rPlotsize.right = m_rPlot.Width();
	m_rPlotsize.bottom = m_rPlot.Height();

	
	if( m_rPlot!=m_rLastPlot )	//if size change then must redo the world
	{
		if( m_MemDC.m_hDC  != NULL )
		{
			m_MemDC.Detach();
			m_MemDC.DeleteDC( );
		}

		m_MemDC.CreateCompatibleDC( pDC );
		m_rLastPlot = m_rPlot;
		
		if( m_bmPlot.m_hObject != NULL )	// delete old bitmap objects
			m_bmPlot.DeleteObject();
//create a new bitmap with current screen DC settings
		m_bmPlot.CreateCompatibleBitmap( pDC, m_rPlotsize.right, m_rPlotsize.bottom );
		m_pDoc->m_LastFilePosition = -1;
	}
	DrawPlot(pDC);
}


//////////////////////////////////////////////////////////////////////
//  Called to draw the specific plot surface when new data is available
//////////////////////////////////////////////////////////////////////
void CPlotData::DrawPlot(CDC* pDC)
{
CBitmap* pbmOld;
//return;
	if(m_pDoc->m_WaveFileOpen)
	{
	// The translate table is used to map DispBuf[] index's into
	//		Screen plot coordinates.
		for(INT i=0; i<m_pDoc->m_NumFFTPts; i++)
		{
			m_TranslateTbl[i] = ( i*m_rPlotsize.bottom )/ m_pDoc->m_NumFFTPts;
		}
		pbmOld = m_MemDC.SelectObject( &m_bmPlot );
		if( m_pDoc->m_CurrentFilePosition != m_pDoc->m_LastFilePosition)
		{
			m_MemDC.SetBkColor( RGB(0,0,0)  );
			// set bitmap background to black using slightly faster ExtTextOut function
			m_MemDC.ExtTextOut( 0, 0,ETO_OPAQUE, &m_rPlotsize,NULL,0,NULL);
			PlotHorzFFTWater( );
			m_pDoc->m_LastFilePosition = m_pDoc->m_CurrentFilePosition;
		}
	// Copy plot bitmap into screen device context for flicker free update
		pDC->BitBlt( m_rPlot.left, m_rPlot.top,
						m_rPlotsize.right,
					m_rPlotsize.bottom, &m_MemDC,
						0,0,SRCCOPY);

	/////////////////////////	
		m_MemDC.SelectObject( pbmOld );	// from memory DC
	}
}


//////////////////////////////////////////////////////////////////////
//		Plot Horizontal Waterfall
//////////////////////////////////////////////////////////////////////
void CPlotData::PlotHorzFFTWater()
{
INT i,x,y,z,yprev;
INT k1;
DWORD prog;
BOOL eof = FALSE;
	k1 = m_rPlotsize.right;
	CBitmap* pbmOld = m_MemDC.SelectObject( &m_bmPlot );
	yprev = 0;
	m_pDoc->m_Wave.SetInPositionSecs(m_pDoc->m_CurrentFilePosition);
	for(x=0; x<k1; x++)
	{
		if(m_pDoc->m_Wave.InRead( m_pDataBuf, FFTSIZE, prog) ==FFTSIZE)
		{
			m_FFT.SendFFT(m_pDataBuf, 0x7FFFFFFF);
			m_FFT.CalcFFT();
			m_FFT.GetFFTData(255,0,m_pDoc->m_NumFFTPts-1,m_pDispBuf);
			for( i=0; i<m_pDoc->m_NumFFTPts; i++ )
			{
				z = 256+m_pDispBuf[i];
				y = m_TranslateTbl[i];	//Convert FFT to display position
				if( y!=yprev )	// still mappped to same y screen pixel coordinate
				{
					m_MemDC.SetPixel( x , y, m_pDoc->m_ColorTbl[z] );
					yprev = y;
				}
			}
		}
		else
		{
			m_pDoc->OpenWaveFile(m_pDoc->m_SrcFilePath);
			m_pDoc->m_Wave.SetInPositionSecs(m_pDoc->m_CurrentFilePosition);
			eof = TRUE;
			break;
		}
	}
	if(eof)
		m_pDoc->m_SegEndPosition = m_pDoc->m_TotalSeconds;
	else
		m_pDoc->m_SegEndPosition = m_pDoc->m_Wave.GetInPositionSecs();
	m_MemDC.SelectObject( pbmOld );
}



void CPlotData::SetFFTParams(BOOL complex, double offset)
{
	m_FFT.SetFFTParams( complex, FFTSIZE, MIN_FFTVALUE, offset, 1, 0,2, FALSE, TRUE, FALSE, 0.0);
}
